require 'date'
require 'logger'
#require 'mushin'
#require 'ssd'
#require 'json' #TODO need to make all responses json based by default if it is a hash and raise an error if it is not a hash!!

require_relative "utter/service"


# Utter defines its interfacing points here. Utter communicate seamlessly 
# via this facade pattern to its internals. This allows all Utter applications 
# makes it easy to have consistant improvments without updating their codebases.
#
#NOTE According to Semantic Versioning 2.0.0 (http://semver.org/); Any changes 
# to this file would require an upgrade to a new Major version. 
module Utter
  # MAJOR version when you make incompatible API changes
  # MINOR version when you add functionality in a backwards-compatible manner 
  # PATCH version when you make backwards-compatible bug fixes.
  VERSION = "1.9.0"

  # UNKNOWN: An unknown message that should always be logged.
  # FATAL: An unhandleable error that results in a program crash.
  # ERROR: A handleable error condition.
  # WARN: A warning.
  # INFO: Generic (useful) information about system operation.
  # DEBUG: Low-level information for developers.
  def self.logger(shift_age: 'daily',
		  datetime_format: '%Y-%m-%d %H:%M:%S',
		  log_dir: 'logs'
		 )

    Dir.mkdir(log_dir) unless File.exists?(log_dir)
    logger_file = "#{log_dir}/#{DateTime.now.strftime('%m_%d_%Y')}.log"
    file = File.open(logger_file, File::WRONLY | File::APPEND | File::CREAT)

    @log = Logger.new("| tee " + file.path, shift_age)
    @log.datetime_format = datetime_format

    @log.info "Utter Log Levels: DEBUG < INFO < WARN < ERROR < FATAL < UNKNOWN"
    return @log
  end


  # Utter choose to provide its functionality via Classes vs Modules as an assertive way of specifing behaviour.

  # Auth is used to build authentication Microservices and is also used utter generated templates.
  #class Auth < Utter::Internals::Auth; end

  # BASIC FEATURE
  #TODO configuring an options {} to define if a microservice is public or private
  # Public::Microservice can be used to provide a microservice that doesn't require authentication.
  #class Public::Microservice < Utter::Internals::Public::Microservice; end
  # Private::Microservice can be used to provide a microservice that DOES require authentication.
  class Service < Utter::Internals::Service; end

  # BASIC FEATURE
  # Usage of non domain-specific stack inside a microservice would be like
  # Stack.new do  
  #   use Trace::Ext,  "A opts hash",  x
  #   use Trace::Ext,  "B opts hash", "B params hash"
  #   use Trace::Ext,  "C opts hash", "C params hash"
  # end
  # Very helpful if you just want to build extenstions conforming to Mushin ducking interface of :initialize(app), :call(env)
  # Yet it is strongly encourged to require your own domain-specific frameworks generated via mushin inexample, and use only
  # a constraining yourself for a single domain-framework per microservice. example
  #
  # require 'domainframeworkXYZ'
  # require 'domainframeworkXYZ_ext_1'
  # require 'domainframeworkXYZ_ext_2'
  # require 'domainframeworkXYZ_ext_3'
  # DomainFrameworkXYZ::Stack do 
  #  use DomainFrameworkXYZ_EXT_1, opts, params
  #  use DomainFrameworkXYZ_EXT_2, opts, params
  #  use DomainFrameworkXYZ_EXT_3, opts, params
  # end
  # this way your work on that domain is ackonwledge\enhanced by the community at large.
  #class Stack < Mushin::Stack
    # Manipulating the Stack
    # Implicitly insert storage via SSD ext (NOONE ELSE NEEDS TO KNOW ABOUT SSD, EXCEPT HERE! BEAUTIFUL!!!!!!)
    # or Explicitly insert it while being in the microservice this way develoeprs get to choose when to use 
    # the right tool for the job. Example SSD for somthing, and Redis for some Caching microservice, etc.
  #end

  # Makes sure that any class that inherits form "< Utter::Ext" must implement :initialize and :call
  #class Ext  < Mushin::Ext
  #end

  class Context
    def initialize &block
      instance_eval &block if block_given?
    end
  end

  class Auth
  end

  # ADVANCED FEATURE
  #TODO expose mushin rule-based capabilities via Utter to provide json rule-based microservices

  # Asserts that Utter::Internals namespace is inaccessible directly for Utter apps.
  private_constant :Internals 
end
